require 'json'
module Bitcointerminal

  module JwtAuth

    private
    def authenticated_get(url, params = {})
      new_rest_connection.get do |req|
        req.url build_url(url)
        params.each do |k,v|
          req.params[k] = v
        end
        req.headers = req.headers = get_headers()
        req.options.timeout = config.rest_timeout
        req.options.open_timeout = config.rest_open_timeout
      end
    end


    def authenticated_post(url, options = {})
      body = options[:params] || {}
      # p "x: #{url} #{body}"
      new_rest_connection.post do |req|
        req.url build_url(url)
        req.headers = get_headers()
        req.body = body.to_json
        req.options.timeout = config.rest_timeout
        req.options.open_timeout = config.rest_open_timeout
      end
    end

    def authenticated_patch(url, options = {})
      # p "authenticated_patch: #{url} #{options}"
      body = options[:params] || {}
      new_rest_connection.patch do |req|
        req.url build_url(url)
        req.headers = get_headers()
        req.body = body.to_json
        req.options.timeout = config.rest_timeout
        req.options.open_timeout = config.rest_open_timeout
      end
    end


    def get_headers()
      headers = {}
      headers['Content-Type'] = 'application/json'
      #syscom
      if config.authorization_token
        headers['authorization'] = config.authorization_token
      else
        #auth0
        bearer = get_bearer_token
        headers['authorization'] = "#{bearer["token_type"]} #{bearer["access_token"]}"
      end

      puts "headers: #{headers}"
      return headers
    end

    def get_bearer_token
      url = URI(config.token_url)
      http = Net::HTTP.new(url.host, url.port)
      http.use_ssl = true
      http.verify_mode = OpenSSL::SSL::VERIFY_NONE
      # http.verify_mode =  OpenSSL::SSL::VERIFY_PEER
      request = Net::HTTP::Post.new(url)
      request["content-type"] = 'application/json'
      request.body = "{\"client_id\":\"#{config.client_id}\",\"client_secret\":\"#{config.client_secret}\",\"audience\":\"#{config.audience}\",\"grant_type\":\"#{config.grant_type}\"}"
      response = http.request(request)
      JSON.parse(response.body)
    end

    def build_url(url)
      # URI.join(config.api_endpoint, url).path
      URI.join(config.api_endpoint, url).request_uri
    end

    def new_rest_connection
      Faraday.new(url: base_api_endpoint) do |conn|
        # conn.use Bitfinex::CustomErrors
        conn.response :logger, Logger.new(STDOUT) , bodies: true  if config.debug_connection
        # conn.request :json
        # conn.response :json, :content_type => /\bjson$/
        # conn.response :logger
        conn.adapter :net_http
      end
    end

    def base_api_endpoint
      # puts "config: #{config.api_endpoint}"
      url = URI.parse(config.api_endpoint)
      "#{url.scheme}://#{url.host}:#{url.port}"
    end

    def fullpath(path, query)
      if query.empty?
        path
      else
        "#{path}?#{query}"
      end
    end

  end

end
